@@ -2,8 +2,9 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -58,6 +59,6 @@ USA.
 (define *current-rgraph*)
 
 (define (rgraph-initial-edges rgraph)
-  (list-transform-positive (rgraph-entry-edges rgraph)
-    (lambda (edge)
-      (node-previous=0? (edge-right-node edge)))))
\ No newline at end of file
+  (filter (lambda (edge)
+	    (node-previous=0? (edge-right-node edge)))
+	  (rgraph-entry-edges rgraph)))
\ No newline at end of file
