@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
     Technology
 
 This file is part of MIT/GNU Scheme.
@@ -28,27 +29,25 @@ USA.
 ;;; package: (runtime advice)
 
 (declare (usual-integrations))
+
+(add-boot-deps! '(runtime dynamic))
 
-(define entry-advice-population)
-(define exit-advice-population)
+(define entry-advice-population (make-population))
+(define exit-advice-population (make-population))
 
-(define (initialize-package!)
-  (set! entry-advice-population (make-population))
-  (set! exit-advice-population (make-population))
-  unspecific)
-
-(define the-arguments)
-(define the-procedure)
-(define the-result)
+(define-deferred advice-continuation (make-unsettable-parameter #f))
+(define-deferred the-arguments (make-unsettable-parameter #f))
+(define-deferred the-procedure (make-unsettable-parameter #f))
+(define-deferred the-result (make-unsettable-parameter #f))
 
 (define (*args*)
-  (list-copy the-arguments))
+  (list-copy (the-arguments)))
 
 (define (*proc*)
-  the-procedure)
+  (the-procedure))
 
 (define (*result*)
-  the-result)
+  (the-result))
 
 (define (get-advice procedure)
   (lambda-advice (procedure-lambda procedure)))
@@ -58,15 +57,15 @@ USA.
     (lambda (original-body state)
       original-body
       (if (not (pair? state))
-	  (error:bad-range-argument *lambda 'LAMBDA-ADVICE))
+	  (error:bad-range-argument *lambda 'lambda-advice))
       (values (car state) (cdr state)))))
 
 (define (make-advice-hook)
   ;; This inserts the actual procedure in a constant list.
-  (make-combination
-   (make-combination (ucode-primitive car)
+  (make-scode-combination
+   (make-scode-combination (ucode-primitive car)
 		     (list (list hook/advised-procedure-wrapper)))
-   (list (make-the-environment))))
+   (list (make-scode-the-environment))))
 
 (define (hook/advised-procedure-wrapper environment)
   (advised-procedure-wrapper environment))
@@ -80,22 +79,22 @@ USA.
       (lambda (original-body state)
 	(call-with-current-continuation
 	 (lambda (continuation)
-	   (fluid-let ((advice-continuation continuation))
-	     (with-restart 'USE-VALUE
+	   (parameterize ((advice-continuation continuation))
+	     (with-restart 'use-value
 		 "Return a value from the advised procedure."
 		 continuation
 		 (lambda ()
 		   (prompt-for-evaluated-expression "Procedure value"))
 	       (lambda ()
 		 (for-each (lambda (advice)
-			     (with-simple-restart 'CONTINUE
+			     (with-simple-restart 'continue
 				 "Continue with advised procedure."
 			       (lambda ()
 				 (advice procedure arguments environment))))
 			   (car state))
 		 (let ((value (scode-eval original-body environment)))
 		   (for-each (lambda (advice)
-			       (with-simple-restart 'CONTINUE
+			       (with-simple-restart 'continue
 				   "Return from advised procedure."
 				 (lambda ()
 				   (advice procedure
@@ -104,8 +103,6 @@ USA.
 					   environment))))
 			     (cdr state))
 		   value))))))))))
-
-(define advice-continuation #f)
 
 ;;;; Advisers
 
@@ -145,9 +142,9 @@ USA.
 	   (let ((entry-advice* (edit-entry entry-advice))
 		 (exit-advice* (edit-exit exit-advice)))
 	     (if (not (eq? entry-advice* entry-advice))
-		 (add-to-population! entry-advice-population *lambda))
+		 (add-new-to-population! entry-advice-population *lambda))
 	     (if (not (eq? exit-advice* exit-advice))
-		 (add-to-population! exit-advice-population *lambda))
+		 (add-new-to-population! exit-advice-population *lambda))
 	     (cons entry-advice* exit-advice*)))))
     (lambda-wrap-body! *lambda
       (lambda (body state receiver)
@@ -254,9 +251,9 @@ USA.
 	(write-truncated
 	 (lambda (object width)
 	   (let ((output (write-to-string object width)))
+	     (write-string (cdr output) port)
 	     (if (car output)
-		 (substring-fill! (cdr output) (- width 3) width #\.))
-	     (write-string (cdr output) port)))))
+		 (write-string "..." port))))))
     (if (default-object? result)
 	(write-string "[Entering " port)
 	(begin
@@ -283,7 +280,7 @@ USA.
 		  (write-args arguments)
 		  (write-string "]" port))
 		(begin
-		  (write-args (list-head arguments 10))
+		  (write-args (take arguments 10))
 		  (newline port)
 		  (write-string "          ...]" port))))))
     (newline port)))
@@ -311,14 +308,14 @@ USA.
 ;;;; Break
 
 (define (break-entry-advice procedure arguments environment)
-  (fluid-let ((the-procedure procedure)
-	      (the-arguments arguments))
+  (parameterize ((the-procedure procedure)
+		 (the-arguments arguments))
     (break-rep environment "Breakpoint on entry" procedure arguments)))
 
 (define (break-exit-advice procedure arguments result environment)
-  (fluid-let ((the-procedure procedure)
-	      (the-arguments arguments)
-	      (the-result result))
+  (parameterize ((the-procedure procedure)
+		 (the-arguments arguments)
+		 (the-result result))
     (break-rep environment "Breakpoint on exit" procedure arguments result))
   result)
 
@@ -328,7 +325,7 @@ USA.
 				      (apply trace-display port info)))
 				   message)
 	      environment
-	      advice-continuation))
+	      (advice-continuation)))
 
 (define (break-entry procedure)
   (advise-entry procedure break-entry-advice))
