@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
     Technology
 
 This file is part of MIT/GNU Scheme.
@@ -29,7 +30,41 @@ USA.
 
 (declare (usual-integrations))
 
+(define scode-lambda-body)
+(define set-scode-lambda-body!)
+(define scode-lambda-bound)
+(define scode-lambda-bound?)
+(define scode-lambda-interface)
+(define scode-lambda-name)
+
+;;; A lambda is an abstract 7-tuple consisting of these elements:
+;;;  name          name of the lambda
+;;;  required      list of symbols, required arguments in order
+;;;  optional      list of symbols, optional arguments in order
+;;;  rest          symbol, rest argument, #F if no rest argument
+;;;  auxiliary     list of auxiliaries to be bound to unassigned
+;;;  declarations  list of declarations for the lexical block
+;;;  body          an expression.  If there are auxiliaries, the body typically
+;;;                begins with the appropriate assignments.
+
+;;; A lambda has a concrete representation of either
+;;; (ucode-type lambda) or (ucode-type extended-lambda),
+;;; auxiliaries are implemented as an `internal' lambda
+;;; of a compound lambda.
+
 (define (initialize-package!)
+  (define ((dispatch-0 op-name clambda-op xlambda-op) *lambda)
+    ((cond ((slambda? *lambda) clambda-op)
+	   ((xlambda? *lambda) xlambda-op)
+	   (else (error:wrong-type-argument *lambda "SCode lambda" op-name)))
+     *lambda))
+
+  (define ((dispatch-1 op-name clambda-op xlambda-op) *lambda arg)
+    ((cond ((slambda? *lambda) clambda-op)
+	   ((xlambda? *lambda) xlambda-op)
+	   (else (error:wrong-type-argument *lambda "SCode lambda" op-name)))
+     *lambda arg))
+
   (lambda-body-procedures clambda/physical-body clambda/set-physical-body!
     (lambda (wrap-body! wrapper-components unwrap-body!
 			unwrapped-body set-unwrapped-body!)
@@ -38,14 +73,6 @@ USA.
       (set! clambda-unwrap-body! unwrap-body!)
       (set! clambda-unwrapped-body unwrapped-body)
       (set! set-clambda-unwrapped-body! set-unwrapped-body!)))
-  (lambda-body-procedures clexpr/physical-body clexpr/set-physical-body!
-    (lambda (wrap-body! wrapper-components unwrap-body!
-			unwrapped-body set-unwrapped-body!)
-      (set! clexpr-wrap-body! wrap-body!)
-      (set! clexpr-wrapper-components wrapper-components)
-      (set! clexpr-unwrap-body! unwrap-body!)
-      (set! clexpr-unwrapped-body unwrapped-body)
-      (set! set-clexpr-unwrapped-body! set-unwrapped-body!)))
   (lambda-body-procedures xlambda/physical-body xlambda/set-physical-body!
     (lambda (wrap-body! wrapper-components unwrap-body!
 			unwrapped-body set-unwrapped-body!)
@@ -55,55 +82,62 @@ USA.
       (set! xlambda-unwrapped-body unwrapped-body)
       (set! set-xlambda-unwrapped-body! set-unwrapped-body!)))
   (set! &lambda-components
-	(dispatch-1 'LAMBDA-COMPONENTS
+	(dispatch-1 'lambda-components
 		    clambda-components
-		    clexpr-components
 		    xlambda-components))
   (set! has-internal-lambda?
-	(dispatch-0 'HAS-INTERNAL-LAMBDA?
+	(dispatch-0 'has-internal-lambda?
 		    clambda-has-internal-lambda?
-		    clexpr-has-internal-lambda?
 		    xlambda-has-internal-lambda?))
+  (set! lambda-arity
+	(dispatch-1 'lambda-arity
+		    slambda-arity
+		    xlambda-arity))
+  (set! scode-lambda-body
+	(dispatch-0 'scode-lambda-body
+		    clambda-unwrapped-body
+		    xlambda-unwrapped-body))
+  (set! scode-lambda-bound
+	(dispatch-0 'scode-lambda-bound
+		    clambda-bound
+		    xlambda-bound))
+  (set! scode-lambda-bound?
+	(dispatch-1 'scode-lambda-bound?
+		    clambda-bound?
+		    xlambda-bound?))
+  (set! lambda-immediate-body
+	(dispatch-0 'lambda-immediate-body
+		    slambda-body
+		    xlambda-body))
+  (set! scode-lambda-interface
+	(dispatch-0 'scode-lambda-interface
+		    slambda-interface
+		    xlambda-interface))
+  (set! scode-lambda-name
+	(dispatch-0 'scode-lambda-name
+		    slambda-name
+		    xlambda-name))
+  (set! lambda-names-vector
+	(dispatch-0 'lambda-names-vector
+		    slambda-names-vector
+		    xlambda-names-vector))
+  (set! lambda-unwrap-body!
+	(dispatch-0 'lambda-unwrap-body!
+		    clambda-unwrap-body!
+		    xlambda-unwrap-body!))
   (set! lambda-wrap-body!
-	(dispatch-1 'LAMBDA-WRAP-BODY!
+	(dispatch-1 'lambda-wrap-body!
 		    clambda-wrap-body!
-		    clexpr-wrap-body!
 		    xlambda-wrap-body!))
   (set! lambda-wrapper-components
-	(dispatch-1 'LAMBDA-WRAPPER-COMPONENTS
+	(dispatch-1 'lambda-wrapper-components
 		    clambda-wrapper-components
-		    clexpr-wrapper-components
 		    xlambda-wrapper-components))
-  (set! lambda-unwrap-body!
-	(dispatch-0 'LAMBDA-UNWRAP-BODY!
-		    clambda-unwrap-body!
-		    clexpr-unwrap-body!
-		    xlambda-unwrap-body!))
-  (set! lambda-body
-	(dispatch-0 'LAMBDA-BODY
-		    clambda-unwrapped-body
-		    clexpr-unwrapped-body
-		    xlambda-unwrapped-body))
-  (set! set-lambda-body!
-	(dispatch-1 'SET-LAMBDA-BODY!
+  (set! set-scode-lambda-body!
+	(dispatch-1 'set-scode-lambda-body!
 		    set-clambda-unwrapped-body!
-		    set-clexpr-unwrapped-body!
 		    set-xlambda-unwrapped-body!))
-  (set! lambda-names-vector
-	(dispatch-0 'LAMBDA-NAME
-		    slambda-names-vector
-		    slexpr-names-vector
-		    xlambda-names-vector))
-  (set! lambda-name
-	(dispatch-0 'LAMBDA-NAME
-		    slambda-name
-		    slexpr-name
-		    xlambda-name))
-  (set! lambda-bound
-	(dispatch-0 'LAMBDA-BOUND
-		    clambda-bound
-		    clexpr-bound
-		    xlambda-bound)))
+  unspecific)
 
 ;;;; Hairy Advice Wrappers
 
@@ -153,50 +187,45 @@ USA.
 	 (set-physical-body! *lambda new-body)))))
 
 (define-integrable (make-wrapper original-body new-body state)
-  (make-comment (vector wrapper-tag original-body state) new-body))
+  (make-scode-comment (vector wrapper-tag original-body state) new-body))
 
 (define (wrapper? object)
-  (and (comment? object)
-       (let ((text (comment-text object)))
+  (and (scode-comment? object)
+       (let ((text (scode-comment-text object)))
 	 (and (vector? text)
 	      (not (zero? (vector-length text)))
 	      (eq? (vector-ref text 0) wrapper-tag)))))
 
 (define wrapper-tag
-  '(LAMBDA-WRAPPER))
+  '(lambda-wrapper))
 
 (define-integrable (wrapper-body wrapper)
-  (comment-expression wrapper))
+  (scode-comment-expression wrapper))
 
 (define-integrable (set-wrapper-body! wrapper body)
-  (set-comment-expression! wrapper body))
+  (set-scode-comment-expression! wrapper body))
 
 (define-integrable (wrapper-state wrapper)
-  (vector-ref (comment-text wrapper) 2))
+  (vector-ref (scode-comment-text wrapper) 2))
 
 (define-integrable (set-wrapper-state! wrapper new-state)
-  (vector-set! (comment-text wrapper) 2 new-state))
+  (vector-set! (scode-comment-text wrapper) 2 new-state))
 
 (define-integrable (wrapper-original-body wrapper)
-  (vector-ref (comment-text wrapper) 1))
+  (vector-ref (scode-comment-text wrapper) 1))
 
 (define-integrable (set-wrapper-original-body! wrapper body)
-  (vector-set! (comment-text wrapper) 1 body))
+  (vector-set! (scode-comment-text wrapper) 1 body))
 
 ;;;; Compound Lambda
 
 (define (make-clambda name required auxiliary body)
-  (make-slambda name
-		required
-		(if (null? auxiliary)
-		    body
-		    (make-combination (make-internal-lambda auxiliary body)
-				      (make-unassigned auxiliary)))))
+  (make-slambda name required (make-auxiliary-lambda auxiliary body)))
 
 (define (clambda-components clambda receiver)
   (slambda-components clambda
     (lambda (name required body)
-      (receiver name required '() '#F  ;;! '()
+      (receiver name required '() '#f
 		(lambda-body-auxiliary body)
 		(clambda-unwrapped-body clambda)))))
 
@@ -206,23 +235,33 @@ USA.
       name
       (append required (lambda-body-auxiliary body)))))
 
+(define (clambda-bound? clambda symbol)
+  (or (slambda-bound? clambda symbol)
+      (auxiliary-bound? (slambda-body clambda) symbol)))
+
 (define (clambda-has-internal-lambda? clambda)
   (lambda-body-has-internal-lambda? (slambda-body clambda)))
 
 (define (lambda-body-auxiliary body)
-  (if (combination? body)
-      (let ((operator (combination-operator body)))
+  (if (scode-combination? body)
+      (let ((operator (scode-combination-operator body)))
 	(if (internal-lambda? operator)
 	    (slambda-auxiliary operator)
 	    '()))
       '()))
 
 (define (lambda-body-has-internal-lambda? body)
-  (and (combination? body)
-       (let ((operator (combination-operator body)))
+  (and (scode-combination? body)
+       (let ((operator (scode-combination-operator body)))
 	 (and (internal-lambda? operator)
 	      operator))))
 
+(define (auxiliary-bound? body symbol)
+  (and (scode-combination? body)
+       (let ((operator (scode-combination-operator body)))
+	 (and (internal-lambda? operator)
+	      (internal-lambda-bound? operator symbol)))))
+
 (define clambda-wrap-body!)
 (define clambda-wrapper-components)
 (define clambda-unwrap-body!)
@@ -235,117 +274,112 @@ USA.
 (define (clambda/set-physical-body! clambda body)
   (set-slambda-body! (or (clambda-has-internal-lambda? clambda) clambda) body))
 
-;;;; Compound Lexpr
-
-(define (make-clexpr name required rest auxiliary body)
-  (make-slexpr name
-	       required
-	       (make-combination
-		(make-internal-lexpr
-		 (list rest)
-		 (if (null? auxiliary)
-		     body
-		     (make-combination (make-internal-lambda auxiliary body)
-				       (make-unassigned auxiliary))))
-		(list (let ((environment (make-the-environment)))
-			(make-combination
-			 (ucode-primitive system-subvector-to-list)
-			 (list environment
-			       (+ (length required) 3)
-			       (make-combination
-				(ucode-primitive system-vector-size)
-				(list environment)))))))))
-
-(define (clexpr-components clexpr receiver)
-  (slexpr-components clexpr
-    (lambda (name required body)
-      (let ((internal (combination-operator body)))
-	(let ((auxiliary (slambda-auxiliary internal)))
-	  (receiver name
-		    required
-		    '()
-		    (car auxiliary)
-		    (append (cdr auxiliary)
-			    (lambda-body-auxiliary (slambda-body internal)))
-		    (clexpr-unwrapped-body clexpr)))))))
-
-(define (clexpr-bound clexpr)
-  (slexpr-components clexpr
-    (lambda (name required body)
-      name
-      (let ((internal (combination-operator body)))
-	(append required
-		(slambda-auxiliary internal)
-		(lambda-body-auxiliary (slambda-body internal)))))))
-
-(define (clexpr-has-internal-lambda? clexpr)
-  (let ((internal (combination-operator (slexpr-body clexpr))))
-    (or (lambda-body-has-internal-lambda? (slambda-body internal))
-	internal)))
-
-(define clexpr-wrap-body!)
-(define clexpr-wrapper-components)
-(define clexpr-unwrap-body!)
-(define clexpr-unwrapped-body)
-(define set-clexpr-unwrapped-body!)
-
-(define (clexpr/physical-body clexpr)
-  (slambda-body (clexpr-has-internal-lambda? clexpr)))
-
-(define (clexpr/set-physical-body! clexpr body)
-  (set-slambda-body! (clexpr-has-internal-lambda? clexpr) body))
-
 ;;;; Extended Lambda
 
-(define-integrable xlambda-type
-  (ucode-type extended-lambda))
+(define (xlambda? object)
+  (object-type? (ucode-type extended-lambda) object))
+
+(define-guarantee xlambda "an extended lambda")
+
+(define (%xlambda-body xlambda)
+  (&triple-first xlambda))
+
+(define (%xlambda-names-vector xlambda)
+  (&triple-second xlambda))
+
+(define (%xlambda-encoded-arity xlambda)
+  (object-datum (&triple-third xlambda)))
+
+(define (xlambda-body xlambda)
+  (guarantee-xlambda xlambda 'xlambda-body)
+  (%xlambda-body xlambda))
+
+(define (xlambda-names-vector xlambda)
+  (guarantee-xlambda xlambda 'xlambda-names-vector)
+  (%xlambda-names-vector xlambda))
+
+(define (xlambda-encoded-arity xlambda)
+  (guarantee-xlambda xlambda 'xlambda-encoded-arity)
+  (%xlambda-encoded-arity xlambda))
+
+(define (encode-xlambda-arity n-required n-optional rest?)
+  (+ n-optional (* 256 (+ n-required (if rest? 256 0)))))
+
+(define (decode-xlambda-arity arity receiver)
+  (let ((qr1 (integer-divide arity 256)))
+    (let ((qr2 (integer-divide (car qr1) 256)))
+      (receiver (cdr qr2)
+		(cdr qr1)
+		(= (car qr2) 1)))))
 
 (define (make-xlambda name required optional rest auxiliary body)
   (&typed-triple-cons
-   xlambda-type
-   (if (null? auxiliary)
-       body
-       (make-combination (make-internal-lambda auxiliary body)
-			 (make-unassigned auxiliary)))
+   (ucode-type extended-lambda)
+   (make-auxiliary-lambda auxiliary body)
    (list->vector
     (cons name (append required optional (if rest (list rest) '()))))
    (make-non-pointer-object
-    (+ (length optional)
-       (* 256
-	  (+ (length required)
-	     (if rest 256 0)))))))
-
-(define-integrable (xlambda? object)
-  (object-type? xlambda-type object))
+    (encode-xlambda-arity (length required) (length optional) rest))))
 
 (define (xlambda-components xlambda receiver)
-  (let ((qr1 (integer-divide (object-datum (&triple-third xlambda)) 256)))
-    (let ((qr2 (integer-divide (car qr1) 256)))
-      (let ((ostart (1+ (cdr qr2))))
-	(let ((rstart (+ ostart (cdr qr1))))
-	  (let ((astart (+ rstart (car qr2)))
-		(bound (&triple-second xlambda)))
+  (guarantee-xlambda xlambda 'xlambda-components)
+  (decode-xlambda-arity
+   (%xlambda-encoded-arity xlambda)
+   (lambda (n-required n-optional rest?)
+      (let ((ostart (1+ n-required)))
+	(let ((rstart (+ ostart n-optional)))
+	  (let ((astart (+ rstart (if rest? 1 0)))
+		(bound (%xlambda-names-vector xlambda)))
 	    (receiver (vector-ref bound 0)
 		      (subvector->list bound 1 ostart)
 		      (subvector->list bound ostart rstart)
-		      (if (zero? (car qr2))
-			  #F ;;!'()
-			  (vector-ref bound rstart))
+		      (if rest?
+			  (vector-ref bound rstart)
+			  #f)
 		      (append
 		       (subvector->list bound astart (vector-length bound))
 		       (lambda-body-auxiliary (&triple-first xlambda)))
 		      (xlambda-unwrapped-body xlambda))))))))
 
-(define (xlambda-names-vector xlambda)
-  (&triple-second xlambda))
+(define (xlambda-arity xlambda offset)
+  (xlambda-components xlambda
+    (lambda (name required optional rest auxiliary decl body)
+      name auxiliary decl body
+      (make-lambda-arity (length required)
+			 (length optional)
+			 rest
+			 offset))))
+
+(define (%xlambda-interface xlambda)
+  (decode-xlambda-arity
+   (%xlambda-encoded-arity xlambda)
+   (lambda (n-required n-optional rest?)
+     (let ((bound (%xlambda-names-vector xlambda)))
+       (make-lambda-list
+	(subvector->list bound 1 (+ n-required 1))
+	(subvector->list bound (+ n-required 1) (+ n-optional n-required 1))
+	(and rest? (vector-ref bound (+ n-optional n-required 1)))
+	'())))))
 
 (define (xlambda-name xlambda)
-  (vector-ref (&triple-second xlambda) 0))
+  (guarantee-xlambda xlambda 'xlambda-name)
+  (vector-ref (%xlambda-names-vector xlambda) 0))
+
+(define (xlambda-interface xlambda)
+  (guarantee-xlambda xlambda 'xlambda-interface)
+  (%xlambda-interface xlambda))
 
 (define (xlambda-bound xlambda)
-  (append (let ((names (&triple-second xlambda)))
+  (guarantee-xlambda xlambda 'xlambda-bound)
+  (append (let ((names (%xlambda-names-vector xlambda)))
 	    (subvector->list names 1 (vector-length names)))
-	  (lambda-body-auxiliary (&triple-first xlambda))))
+	  (lambda-body-auxiliary (%xlambda-body xlambda))))
+
+(define (xlambda-bound? xlambda symbol)
+  (guarantee-xlambda xlambda 'xlambda-bound?)
+  (or (let ((bound (%xlambda-names-vector xlambda)))
+	(subvector-find-next-element bound 1 (vector-length bound) symbol))
+      (auxiliary-bound? (%xlambda-body xlambda) symbol)))
 
 (define (xlambda-has-internal-lambda? xlambda)
   (lambda-body-has-internal-lambda? (&triple-first xlambda)))
@@ -370,12 +404,13 @@ USA.
 
 ;;;; Generic Lambda
 
-(define (lambda? object)
+(define (scode-lambda? object)
   (or (slambda? object)
-      (slexpr? object)
       (xlambda? object)))
+(register-predicate! scode-lambda? 'scode-lambda)
 
-(define (make-lambda name required optional rest auxiliary declarations body)
+(define (make-scode-lambda name required optional rest auxiliary declarations
+			   body)
   (let ((interface (append required optional (if rest (list rest) '()))))
     (let ((dup-interface (find-list-duplicates interface))
 	  (dup-auxiliary (find-list-duplicates auxiliary)))
@@ -390,29 +425,32 @@ USA.
   (let ((body*
 	 (if (null? declarations)
 	     body
-	     (make-sequence (list (make-block-declaration declarations)
-				  body)))))
+	     (make-scode-sequence
+	      (list (make-scode-block-declaration declarations)
+		    body)))))
     (cond ((and (< (length required) 256)
 		(< (length optional) 256)
 		(or (not (null? optional))
-		    rest
-		    (not (null? auxiliary))))
+		    rest))
 	   (make-xlambda name required optional rest auxiliary body*))
 	  ((not (null? optional))
-	   (error "Optionals not implemented" 'MAKE-LAMBDA))
+	   (error "Optionals not implemented" 'make-lambda))
 	  (rest
-	   (make-clexpr name required rest auxiliary body*))
+	   (error "You want how many arguments?  AND a rest arg?"))
 	  (else
 	   (make-clambda name required auxiliary body*)))))
 
-(define (lambda-components *lambda receiver)
+(define (scode-lambda-components *lambda receiver)
   (&lambda-components *lambda
     (lambda (name required optional rest auxiliary body)
-      (let ((actions (and (sequence? body) (sequence-actions body))))
-	(if (and actions (block-declaration? (car actions)))
+      (let ((actions
+	     (and (scode-sequence? body)
+		  (scode-sequence-actions body))))
+	(if (and (pair? actions)
+		 (scode-block-declaration? (car actions)))
 	    (receiver name required optional rest auxiliary
-		      (block-declaration-text (car actions))
-		      (make-sequence (cdr actions)))
+		      (scode-block-declaration-text (car actions))
+		      (make-scode-sequence (cdr actions)))
 	    (receiver name required optional rest auxiliary '() body))))))
 
 (define (find-list-duplicates items)
@@ -426,115 +464,131 @@ USA.
 	  (else
 	   (loop (cdr items) duplicates)))))
 
-(define ((dispatch-0 op-name clambda-op clexpr-op xlambda-op) *lambda)
-  ((cond ((slambda? *lambda) clambda-op)
-	 ((slexpr? *lambda) clexpr-op)
-	 ((xlambda? *lambda) xlambda-op)
-	 (else (error:wrong-type-argument *lambda "SCode lambda" op-name)))
-   *lambda))
-
-(define ((dispatch-1 op-name clambda-op clexpr-op xlambda-op) *lambda arg)
-  ((cond ((slambda? *lambda) clambda-op)
-	 ((slexpr? *lambda) clexpr-op)
-	 ((xlambda? *lambda) xlambda-op)
-	 (else (error:wrong-type-argument *lambda "SCode lambda" op-name)))
-   *lambda arg))
 
 (define &lambda-components)
 (define has-internal-lambda?)
+(define lambda-arity)
 (define lambda-wrap-body!)
 (define lambda-wrapper-components)
 (define lambda-unwrap-body!)
-(define lambda-body)
-(define set-lambda-body!)
+(define lambda-immediate-body)
 (define lambda-names-vector)
-(define lambda-name)
-(define lambda-bound)
-
-(define-structure (block-declaration
-		   (type vector)
-		   (named ((ucode-primitive string->symbol)
-			   "#[Block Declaration]")))
-  (text #f read-only #t))
 
-;;;; Simple Lambda/Lexpr
-
-(define-integrable slambda-type
-  (ucode-type lambda))
-
-(define-integrable (make-slambda name required body)
-  (&typed-pair-cons slambda-type body (list->vector (cons name required))))
-
-(define-integrable (slambda? object)
-  (object-type? slambda-type object))
-
-(define (slambda-components slambda receiver)
-  (let ((bound (&pair-cdr slambda)))
-    (receiver (vector-ref bound 0)
-	      (subvector->list bound 1 (vector-length bound))
-	      (&pair-car slambda))))
-
-(define (slambda-names-vector slambda)
-  (&pair-cdr slambda))
-
-(define-integrable (slambda-name slambda)
-  (vector-ref (&pair-cdr slambda) 0))
+;;;; Simple Lambda
+(define (slambda-arity slambda offset)
+  (guarantee-slambda slambda 'slambda-arity)
+  (%slambda-arity slambda offset))
 
 (define (slambda-auxiliary slambda)
-  (let ((bound (&pair-cdr slambda)))
-    (subvector->list bound 1 (vector-length bound))))
+  (guarantee-slambda slambda 'slambda-auxiliary)
+  (%slambda-auxiliary slambda))
 
-(define-integrable (slambda-body slambda)
+(define (slambda-body slambda)
+  (guarantee-slambda slambda 'slambda-body)
+  (%slambda-body slambda))
+
+(define (set-slambda-body! slambda new-body)
+  (guarantee-slambda slambda 'set-slambda-body!)
+  (%set-slambda-body! slambda new-body))
+
+(define (slambda-components slambda receiver)
+  (guarantee-slambda slambda 'slambda-components)
+  (%slambda-components slambda receiver))
+
+(define (slambda-interface slambda)
+  (guarantee-slambda slambda 'slambda-interface)
+  (%slambda-interface slambda))
+
+(define (slambda-name slambda)
+  (guarantee-slambda slambda 'slambda-name)
+  (%slambda-name slambda))
+
+(define (slambda-names-vector slambda)
+  (guarantee-slambda slambda 'slambda-names-vector)
+  (%slambda-names-vector slambda))
+
+(define (make-slambda name required body)
+  (&typed-pair-cons (ucode-type lambda)
+		    body (list->vector (cons name required))))
+
+(define-integrable (slambda? object)
+  (object-type? (ucode-type lambda) object))
+
+(define-guarantee slambda "simple lambda")
+
+(define-integrable (%slambda-body slambda)
   (&pair-car slambda))
 
-(define-integrable (set-slambda-body! slambda body)
+(define-integrable (%set-slambda-body! slambda body)
   (&pair-set-car! slambda body))
 
-(define-integrable slexpr-type
-  (ucode-type lexpr))
+(define-integrable (%slambda-names-vector slambda)
+  (&pair-cdr slambda))
 
-(define-integrable (make-slexpr name required body)
-  (&typed-pair-cons slexpr-type body (list->vector (cons name required))))
+(define (%slambda-arity slambda offset)
+  (make-lambda-arity
+   (- (vector-length (%slambda-names-vector slambda)) 1)
+   0
+   #f
+   offset))
 
-(define-integrable (slexpr? object)
-  (object-type? slexpr-type object))
+(define-integrable (%slambda-auxiliary slambda)
+  (let ((bound (%slambda-names-vector slambda)))
+    (subvector->list bound 1 (vector-length bound))))
 
-(define (slexpr-components slexpr receiver)
-  (let ((bound (&pair-cdr slexpr)))
-    (receiver (vector-ref bound 0)
-	      (subvector->list bound 1 (vector-length bound))
-	      (&pair-car slexpr))))
+(define-integrable (%slambda-interface slambda)
+  (let ((bound (%slambda-names-vector slambda)))
+    (make-lambda-list
+     (subvector->list bound 1 (vector-length bound))
+     '()
+     #f
+     '())))
 
-(define (slexpr-names-vector slexpr)
-  (&pair-cdr slexpr))
+(define (slambda-bound? slambda symbol)
+  (let ((bound (%slambda-names-vector slambda)))
+    (subvector-find-next-element bound 1 (vector-length bound) symbol)))
 
-(define-integrable (slexpr-name slexpr)
-  (vector-ref (&pair-cdr slexpr) 0))
+(define-integrable (%slambda-name slambda)
+  (vector-ref (%slambda-names-vector slambda) 0))
 
-(define-integrable (slexpr-body slexpr)
-  (&pair-car slexpr))
+(define (%slambda-components slambda receiver)
+  (receiver (%slambda-name slambda)
+	    (%slambda-interface slambda)
+	    (%slambda-body slambda)))
 
 ;;;; Internal Lambda
 
-(define-integrable lambda-tag:internal-lambda
-  ((ucode-primitive string->symbol) "#[internal-lambda]"))
+(define-integrable (%make-internal-lambda names body)
+  (make-slambda scode-lambda-name:internal-lambda names body))
 
-(define-integrable lambda-tag:internal-lexpr
-  ((ucode-primitive string->symbol) "#[internal-lexpr]"))
-
-(define-integrable (make-internal-lambda names body)
-  (make-slambda lambda-tag:internal-lambda names body))
-
-(define-integrable (make-internal-lexpr names body)
-  (make-slambda lambda-tag:internal-lexpr names body))
+(define (make-auxiliary-lambda auxiliary body)
+  (if (null? auxiliary)
+      body
+      (make-scode-combination (%make-internal-lambda auxiliary body)
+			      (make-unassigned auxiliary))))
 
 (define (internal-lambda? *lambda)
   (and (slambda? *lambda)
-       (or (eq? (slambda-name *lambda) lambda-tag:internal-lambda)
-	   (eq? (slambda-name *lambda) lambda-tag:internal-lexpr))))
+       (eq? (slambda-name *lambda) scode-lambda-name:internal-lambda)))
+
+(define (internal-lambda-bound? *lambda symbol)
+  (and (slambda? *lambda)
+       (slambda-bound? *lambda symbol)))
 
 (define (make-unassigned auxiliary)
   (map (lambda (auxiliary)
 	 auxiliary
 	 (make-unassigned-reference-trap))
-       auxiliary))
\ No newline at end of file
+       auxiliary))
+
+(define (make-lambda-arity required-count optional-count rest? offset)
+  (let ((r (fix:- required-count offset)))
+    (cond (rest?
+	   (make-procedure-arity (fix:max 0 r) #f))
+	  ((fix:>= r 0)
+	   (make-procedure-arity r (fix:+ r optional-count)))
+	  (else
+	   (error "Illegal arity for entity:"
+		  (list required-count optional-count rest? offset))))))
+
+(initialize-package!)
\ No newline at end of file
