@@ -3,7 +3,8 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -29,6 +30,11 @@ USA.
 
 (declare (usual-integrations))
 
+(add-boot-deps! '(runtime number))
+(add-boot-init!
+ (lambda ()
+   (add-pathname-host-type! 'unix make-unix-host-type)))
+
 (define (make-unix-host-type index)
   (make-host-type index
 		  'unix
@@ -45,9 +51,6 @@ USA.
 		  unix/user-homedir-pathname
 		  unix/init-file-pathname
 		  unix/pathname-simplify))
-
-(define (initialize-package!)
-  (add-pathname-host-type! 'unix make-unix-host-type))
 
 ;;;; Pathname Parser
 
@@ -317,7 +320,7 @@ USA.
   (if (pair? (pathname-directory pathname))
       (let loop ((pathname pathname) (np 1))
 	(let ((directory (pathname-directory pathname)))
-	  (let scan ((p (list-tail directory np)) (np np))
+	  (let scan ((p (drop directory np)) (np np))
 	    (if (pair? p)
 		(cond ((and (not (eq? (car p) 'up))
 			    (pair? (cdr p))
@@ -353,5 +356,5 @@ USA.
 
 (define (file-eq? p1 p2)
   ((ucode-primitive file-eq? 2)
-   (string-for-primitive (->namestring (merge-pathnames p1)))
-   (string-for-primitive (->namestring (merge-pathnames p2)))))
\ No newline at end of file
+   (->namestring (merge-pathnames p1))
+   (->namestring (merge-pathnames p2))))
\ No newline at end of file
